from odoo import api, fields, models


class AccountInvoice(models.Model):
    _inherit = "account.invoice"

    amount_due_discount = fields.Monetary(
        string='Amount Due (Discounted)',
        compute='_compute_amount_due_discount',
        help="If the invoice is eligible for a discount (based on "
             "its Payment Terms), this is the current payoff amount.")
    display_amount_due_discount = fields.Boolean(
        string='Display "Amount Due (Discounted)"?',
        compute='_compute_display_amount_due_discount')

    @api.multi
    def _compute_amount_due_discount(self):
        """
        Determine if a given invoice is eligible for a cash discount based on
        its Payment Term details and what is the payoff amount. If the invoice
        is not eligible for a discount, the payoff amount is just `residual`.

        Notes about `payment_term_details` variable:
            `compute` returns a list where each item is "Date Due, Amount Due".

        Example result of compute on a $100.00 invoice where the two lines of a
        payment term are 97% due in 10 days and the balance is due in 30 days.
            [[('2018-01-10', 97.0), ('2018-01-30', 3.0)]]

        :return None:
        """
        date_ref = fields.Date.today()
        for invoice in self:
            # If the invoice is ineligible for discount for any reason,
            # Amount Due (Discounted) will fallback to the standard Amount Due.
            amount_due = invoice.residual
            # If amounts are different don't do anything (for now)
            # TODO: Test with different "Amount Due", such as previously
            # applied partial payments
            if amount_due and amount_due == invoice.amount_total:
                payment_term = invoice.payment_term_id
                # See method docstrings about `payment_term_details`
                payment_term_details = payment_term.compute(invoice.residual,
                                                            date_ref=invoice.date_invoice)
                if payment_term_details:
                    payment_term_lines = payment_term.line_ids
                    for i, detail in enumerate(payment_term_details[0]):
                        if i + 1 == len(payment_term_details[0]):
                            # If this is the last detail line, force balance as amount due
                            amount_due = invoice.residual
                        else:
                            date_maturity = detail[0]
                            amount_due = detail[1]
                            # Confirm this Payment Term Line is supposed to discount
                            # the remainder and it is before the Due Date.
                            if payment_term_lines[i].discount_if_paid and date_ref <= date_maturity:
                                amount_due = amount_due
                                # We found the amount due, so exit the loop
                                break

            invoice.amount_due_discount = amount_due

    @api.multi
    def _compute_display_amount_due_discount(self):
        """
        Determine if the "Amount Due (Discounted)" field should be displayed on
        the invoice. It will only be shown if the "Amount Due (Discounted)" is
        different than the "Amount Due" and it's not equal to zero.

        :return None:
        """
        for inv in self:
            inv.display_amount_due_discount = inv.amount_due_discount and inv.amount_due_discount != inv.residual
