from odoo import api, fields, models
from odoo.addons.account.models.account_payment import MAP_INVOICE_TYPE_PARTNER_TYPE


class AccountRegisterPayments(models.TransientModel):
    """
    Inherit the Register Payment Wizard to allow tracking amount of payment
    difference and provide the user the choice to writeoff the difference. The
    majority of features are ported from the core `account.payment` model with
    minimal changes.
    """
    _inherit = "account.register.payments"

    payment_difference = fields.Monetary(compute='_compute_payment_difference',
                                         readonly=True)
    payment_difference_handling = fields.Selection([('open', 'Keep open'),
                                                    ('reconcile', 'Mark invoice as fully paid')],
                                                   string="Payment Difference",
                                                   default='open',
                                                   copy=False)
    writeoff_account_id = fields.Many2one('account.account',
                                          string='Difference Account',
                                          domain=[('deprecated', '=', False)],
                                          copy=False)

    @api.one
    @api.depends('amount', 'payment_date', 'currency_id')
    def _compute_payment_difference(self):
        """
        Determine the amount of difference from the invoices' total amount and
        the amount being paid on the register payments wizard.model

        Copied from core's `account.payment` model. No changes made except to
        remove dependency on `invoice_ids` which doesn't exist on this model.

        :return None:
        """
        context = dict(self._context or {})
        active_model = context.get('active_model')
        active_ids = context.get('active_ids')
        invoices = self.env[active_model].browse(active_ids)
        if not invoices:
            return
        if invoices[0].type in ['in_invoice', 'out_refund']:
            self.payment_difference = self.amount - self._compute_total_invoices_amount()
        else:
            self.payment_difference = self._compute_total_invoices_amount() - self.amount

    @api.model
    def default_get(self, fields):
        """
        Inherit `super` to modify the default values for the Register Payment
        form. If the invoice is eligible for a discount (based on its
        Payment Terms), default these values:
          - Payment Amount: Amount Due (Discounted)
          - Payment Difference: Mark invoice as fully paid
          - Post Difference in: Default Payment Difference Account for invoice type

        :param list fields: fields for which to get default values
        :return dict: default fields/values for the payment
        """
        rec = super(AccountRegisterPayments, self).default_get(fields)
        context = dict(self._context or {})
        active_model = context.get('active_model')
        active_ids = context.get('active_ids')
        invoices = self.env[active_model].browse(active_ids)

        amount_due_discount = 0.0
        residual = 0.0
        for invoice in invoices:
            sign = -1 if invoice.type in ['in_refund', 'out_refund'] else 1
            amount_due_discount += abs(invoice.amount_due_discount) * sign
            residual = abs(invoice.residual) * sign
        rec.update({
            'amount': amount_due_discount,
            'communication': ", ".join([inv.reference or inv.name or inv.number
                                        for inv in invoices]),
        })

        if amount_due_discount != residual:
            partner_type = MAP_INVOICE_TYPE_PARTNER_TYPE.get(invoices[0].type)
            rec.update({
                'payment_difference_handling': 'reconcile'
                                               if self.payment_difference != 0
                                               else 'open',
                'writeoff_account_id': self.env['ir.default'].get(
                    model_name='account.payment',
                    field_name='writeoff_account_id',
                    company_id=self.env.user.id,
                    condition=partner_type)
            })

        return rec

    def get_payment_vals(self):
        """
        Extend super to default our custom fields onto the generated payment.

        :return dict: fields/values to use for payment creation
        """
        res = super(AccountRegisterPayments, self).get_payment_vals()
        res.update({
            'payment_difference': self.payment_difference,
            'payment_difference_handling': self.payment_difference_handling,
            'writeoff_account_id': self.writeoff_account_id.id,
        })
        return res
